# generated by datamodel-codegen:
#   filename:  disk.json

from __future__ import annotations

from pydantic import BaseModel, Extra, Field


class LastUpdated(BaseModel):
    """
    Last updated
    """

    class Config:
        extra = Extra.allow

    devices: float
    io_counters_read_count: float
    io_counters_write_count: float
    io_counters_read_bytes: float
    io_counters_write_bytes: float
    io_counters_read_time: float
    io_counters_write_time: float
    partitions: float


class Disk(BaseModel):
    """
    Disk
    """

    class Config:
        extra = Extra.allow

    devices: list
    io_counters_read_count: int
    io_counters_write_count: int
    io_counters_read_bytes: int
    io_counters_write_bytes: int
    io_counters_read_time: int
    io_counters_write_time: int
    partitions: list
    last_updated: LastUpdated = Field(..., description="Last updated")
